@@ -3,10 +3,6 @@
 #ifndef NGX_HTTP_LUA_COMMON_H
 #define NGX_HTTP_LUA_COMMON_H
 
-#ifndef DDEBUG
-#define DDEBUG 0
-#endif
-
 #include "ddebug.h"
 
 #include <nginx.h>
@@ -29,6 +25,11 @@
 #define MD5_DIGEST_LENGTH 16
 #endif
 
+#define NGX_HTTP_LUA_CHECK_ABORTED(L, ctx) \
+        if (ctx && ctx->aborted) { \
+            return luaL_error(L, "coroutine aborted"); \
+        }
+
 /* Nginx HTTP Lua Inline tag prefix */
 
 #define NGX_HTTP_LUA_INLINE_TAG "nhli_"
@@ -58,6 +59,16 @@ typedef struct {
 #endif
 
 
+#ifndef NGX_HTTP_LUA_MAX_ARGS
+#define NGX_HTTP_LUA_MAX_ARGS 100
+#endif
+
+
+#ifndef NGX_HTTP_LUA_MAX_HEADERS
+#define NGX_HTTP_LUA_MAX_HEADERS 100
+#endif
+
+
 typedef struct {
     lua_State       *lua;
 
@@ -147,6 +158,8 @@ typedef struct {
     ngx_uint_t       index; /* index of the current subrequest in its
                                parent request */
 
+    unsigned         waiting;  /* number of subrequests being waited */
+
     ngx_str_t        exec_uri;
     ngx_str_t        exec_args;
 
@@ -157,10 +170,7 @@ typedef struct {
                                             0: header not sent yet */
 
     unsigned       eof:1;             /*  1: last_buf has been sent;
-                                            0: last_buf not sent yet */
-
-    unsigned       waiting;         /*  1: subrequest is still running;
-                                            0: subrequest is not running */
+                                          0: last_buf not sent yet */
 
     unsigned       done:1;            /*  1: subrequest is just done;
                                             0: subrequest is not done
@@ -188,6 +198,7 @@ typedef struct {
     unsigned         req_header_cached:1;
 
     unsigned         waiting_flush:1;
+    unsigned         aborted:1;
 } ngx_http_lua_ctx_t;
 
 
